
/*
  Name			: libsmpp
  Copyright		: Daniel Sirait
  Author		: Daniel Sirait
  Date			: 23/08/10 10:02
  Description	: SMPP Library
*/



#ifndef _SMPP_TYPE_H
#define _SMPP_TYPE_H

#ifdef __cplusplus
extern "C" {
#endif

typedef unsigned char byte;
typedef unsigned short word;

/* PDU Header - mandatory (big endian) */
typedef struct _PDU_HEADER {
	unsigned long command_len;
	unsigned long command_id;   		// 0x00000000 to 0x000001FF
	unsigned long command_status;       // success or failure status, error code
	unsigned long seq_num;      		// sequence_number, 0x00000001 to 0x7FFFFFFF.
	
} PDU_HEADER, *pPDU_HEADER;


/* PDU Body - optional */
typedef struct _PDU_BODY {
	byte *content;
} PDU_BODY, *pPDU_BODY;

typedef struct _TLV {
	word tag;
	word length;
	byte *value;
} TLV, *pTLV;

typedef struct _PDU {
	PDU_HEADER header;
	PDU_BODY body;
} PDU, *pPDU;



/* BIND operation */

typedef struct _BIND_TRANSMITTER {
	PDU_HEADER header;
	char system_id[16];
	char password[9];
	char system_type[13];
	byte interface_version;
	byte addr_ton;
	byte addr_npi;
	char address_range[41];
	
} BIND_TRANSMITTER;

typedef struct _BIND_RECEIVER {
	PDU_HEADER header;
	char system_id[16];
	char password[9];
	char system_type[13];
	byte interface_version;
	byte addr_ton;
	byte addr_npi;
	char address_range[41];
} BIND_RECEIVER;

typedef struct _BIND_RECEIVER_RESP {
	PDU_HEADER header;
	char system_id[16];
	TLV sc_interface_version;
} BIND_RECEIVER_RESP;

typedef struct _OUTBIND {
	PDU_HEADER header;
	char system_id[16];
	char password[9];
} OUTBIND;


/* UNBIND operation */

typedef struct _UNBIND {
	PDU_HEADER header;			// command_id field must include the Command ID value corresponding to the unbind operation.
	
} UNBIND;

// The SMPP unbind_resp PDU is used to reply to an unbind request. It comprises the SMPP
// message header only.
typedef struct _UNBIND_RESP {
    PDU_HEADER header;

} UNBIND_RESP;


/* GENERIC_NACK */
typedef struct _GENERIC_NACK {
	PDU_HEADER header;
} GENERIC_NACK;


typedef struct _SUBMIT_SM {
	PDU_HEADER header;			// .command_status = NULL
	char service_type[6];		// set to NULL = 0x00
	byte source_addr_ton;
	byte source_addr_npi;
	char source_addr[21];
	byte dest_addr_ton;
	byte dest_addr_npi;
	char destination_addr[21];
	byte esm_class;
	byte protocol_id;
	byte priority_flag;
	char schedule_delivery_time[17];        // 1 or 17 ?
	char validity_period[17];
	byte registered_delivery;
	byte replace_if_present_flag;
	byte data_coding;
	byte sm_default_msg_id;
	byte sm_length;
	char short_message[254];
	struct {
        TLV user_message_reference;
        TLV source_port;
        TLV source_addr_subunit;
        TLV destination_port;
        TLV dest_addr_subunit;
        TLV sar_msg_ref_num;
        TLV sar_total_segment;
        TLV sar_segment_seqnum;
        TLV more_messages_to_send;
        TLV payload_type;
        TLV message_payload;
        TLV privacy_indicator;
        TLV callback_num;
        TLV callback_num_pres_ind;
        TLV callback_num_atag;
        TLV source_subaddress;
        TLV dest_subaddress;
        TLV user_response_code;
        TLV display_time;
        TLV sms_signal;
        TLV ms_validity;
        TLV ms_msg_wait_facilities;
        TLV number_of_message;
        TLV alert_on_msg_delivery;
        TLV language_indicator;
        TLV its_reply_type;
        TLV its_session_info;
        TLV ussd_service_op;
	} optional;
} SUBMIT_SM;

typedef struct _SUBMIT_SM_RESP {
	PDU_HEADER header;
	char message_id[65];		// max 65 ! , 9 33 65
} _SUBMIT_SM_RESP;


typedef struct _SUBMIT_MULTI {
	PDU_HEADER header;
	char service_type[6];
	unsigned long source_addr_ton;
	unsigned long source_addr_npi;
	char source_addr[21];		// ESME can be NULL, source_addr_ton, source_addr_npi, source_addr
	unsigned long number_of_dests;	// indicates the number of dest_address structures that are to follow. A maximum of 254
	//char dest_address[24];		// Var. n[2-24], dest_addr_ton, dest_addr_npi and destination_addr
	/*
	struct _dest_address {
		byte dest_flag;
		
		struct _SME_dest_address {
			dest_addr_ton
			dest_addr_npi
			destination_addr
		} SME_dest_address;
		struct _DL_Name {
			char dl_name[21];
		} DL_Name;
	} dest_address;
	*/
	void *dest_address;			// allocate with malloc 
	unsigned long protocol_id;
	unsigned long priority_flag;
	char schedule_delivery_time[17];	// 1 or 17
	char validity_period[17];		// 1 or 17
	unsigned long registered_delivery;
	unsigned long replace_if_present_flag;
	unsigned long data_coding;
	unsigned long sm_default_msg_id;
	unsigned long sm_length;
	char short_message[254];		// 0-254
	struct _optional {
        TLV user_message_reference;	
        TLV source_port;
        TLV source_addr_subunit;
        TLV destination_port;
        TLV dest_addr_subunit;
        TLV sar_msg_ref_num;
        TLV sar_total_segment;
        TLV sar_segment_seqnum;
        //TLV more_messages_to_send;
        TLV payload_type;
        TLV message_payload;
        TLV privacy_indicator;
        TLV callback_num;
        TLV callback_num_pres_ind;
        TLV callback_num_atag;
        TLV source_subaddress;
        TLV dest_subaddress;
        //TLV user_response_code;
        TLV display_time;
        TLV sms_signal;
        TLV ms_validity;
        TLV ms_msg_wait_facilities;
        //TLV number_of_message;
        TLV alert_on_msg_delivery;
        TLV language_indicator;
        //TLV its_reply_type;
        //TLV its_session_info;
        //TLV ussd_service_op;
	} optional;
	
} SUBMIT_MULTI;

typedef struct _SUBMIT_MULTI_RESP {
	PDU_HEADER header;
	char message_id[65];
	char no_unsuccess;
	struct _unsuccess_sme{
		unsigned long dest_addr_ton;
		unsigned long dest_addr_npi;
		char destination_addr[21];
		unsigned long error_status_code;
		
	} unsuccess_sme;		// unsuccess_sme(s)
	
} SUBMIT_MULTI_RESP;





#ifdef __cplusplus
}
#endif

#endif
